@@ -6,27 +6,32 @@ from __future__ import absolute_import
 
 import logging
 
+from pyparsing import ParseException
+
 from base64 import b64encode, b64decode
 from sqlalchemy import event
 from sqlalchemy.engine import create_engine, Engine
+from sqlalchemy.exc import ProgrammingError
 from sqlalchemy.orm.exc import NoResultFound
 from sqlalchemy.sql.expression import and_
 
+from tiddlyweb.filters import FilterIndexRefused
 from tiddlyweb.model.bag import Bag
 from tiddlyweb.model.policy import Policy
 from tiddlyweb.model.recipe import Recipe
 from tiddlyweb.model.tiddler import Tiddler
 from tiddlyweb.model.user import User
-from tiddlyweb.serializer import Serializer
 from tiddlyweb.store import (NoBagError, NoRecipeError, NoTiddlerError,
-        NoUserError)
+        NoUserError, StoreError)
 from tiddlyweb.stores import StorageInterface
 from tiddlyweb.util import binary_tiddler
 
 from .model import (Base, Session, sBag, sPolicy, sRecipe, sTiddler, sRevision,
         sText, sTag, sField, sCurrentRevision, sFirstRevision, sUser, sRole)
+from .parser import DEFAULT_PARSER
+from .producer import Producer
 
-__version__ = '3.0.16'
+__version__ = '3.1.0'
 
 #logging.basicConfig()
 #logging.getLogger('sqlalchemy.engine').setLevel(logging.INFO)
@@ -43,6 +48,9 @@ class Store(StorageInterface):
     def __init__(self, store_config=None, environ=None):
         super(Store, self).__init__(store_config, environ)
         self.store_type = self._db_config().split(':', 1)[0]
+        self.parser = DEFAULT_PARSER
+        self.producer = Producer()
+        self.has_geo = False
         self._init_store()
 
     def _init_store(self):
@@ -54,7 +62,6 @@ class Store(StorageInterface):
         Base.metadata.bind = engine
         Session.configure(bind=engine)
         self.session = Session()
-        self.serializer = Serializer('text')
 
         # turn on foreign keys for sqlite (the default engine)
         if 'sqlite' in self.store_type:
@@ -170,7 +177,7 @@ class Store(StorageInterface):
 
     def recipe_put(self, recipe):
         try:
-            srecipe = self._store_recipe(recipe)
+            self._store_recipe(recipe)
             self.session.commit()
         except:
             self.session.rollback()
@@ -206,7 +213,7 @@ class Store(StorageInterface):
 
     def bag_put(self, bag):
         try:
-            sbag = self._store_bag(bag)
+            self._store_bag(bag)
             self.session.commit()
         except:
             self.session.rollback()
@@ -267,7 +274,7 @@ class Store(StorageInterface):
             if not tiddler.bag:
                 raise NoBagError('bag required to save')
             try:
-                sbag = self.session.query(sBag.id).filter(sBag.name
+                self.session.query(sBag.id).filter(sBag.name
                         == tiddler.bag).one()
             except NoResultFound, exc:
                 raise NoBagError('bag %s must exist for tiddler save: %s'
@@ -319,6 +326,42 @@ class Store(StorageInterface):
             self.session.rollback()
             raise
 
+    def search(self, search_query=''):
+        """
+        Do a search of of the database, using the 'q' query,
+        parsed by the parser and turned into a producer.
+        """
+        query = self.session.query(sTiddler).join('current')
+        config = self.environ.get('tiddlyweb.config', {})
+        if '_limit:' not in search_query:
+            default_limit = config.get('mysql.search_limit',
+                    config.get('sqlalchemy3.search_limit', '20'))
+            search_query += ' _limit:%s' % default_limit
+        try:
+            try:
+                ast = self.parser(search_query)[0]
+                fulltext = config.get('mysql.fulltext', False)
+                query = self.producer.produce(ast, query, fulltext=fulltext,
+                        geo=self.has_geo)
+            except ParseException, exc:
+                raise StoreError('failed to parse search query: %s' % exc)
+
+            try:
+                for stiddler in query.all():
+                    try:
+                        yield Tiddler(unicode(stiddler.title),
+                                unicode(stiddler.bag))
+                    except AttributeError:
+                        stiddler = stiddler[0]
+                        yield Tiddler(unicode(stiddler.title),
+                                unicode(stiddler.bag))
+                self.session.close()
+            except ProgrammingError, exc:
+                raise StoreError('generated search SQL incorrect: %s' % exc)
+        except:
+            self.session.rollback()
+            raise
+
     def _load_bag(self, bag, sbag):
         bag.desc = sbag.desc
         bag.policy = self._load_policy(sbag.policy)
@@ -522,3 +565,33 @@ class Store(StorageInterface):
         suser.password = user._password
         suser.note = user.note
         return suser
+
+
+def index_query(environ, **kwargs):
+    """
+    Attempt to optimize filter processing by using the search index
+    to provide results that can be matched.
+
+    In practice, this proves not to be that helpful when memcached
+    is being used, but it is in other situations.
+    """
+    store = environ['tiddlyweb.store']
+
+    queries = []
+    for key, value in kwargs.items():
+        if '"' in value:
+            # XXX The current parser is currently unable to deal with
+            # nested quotes. Rather than running the risk of tweaking
+            # the parser with unclear results, we instead just refuse
+            # for now. Later this can be fixed for real.
+            raise FilterIndexRefused('unable to process values with quotes')
+        queries.append('%s:"%s"' % (key, value))
+    query = ' '.join(queries)
+
+    storage = store.storage
+
+    try:
+        tiddlers = storage.search(search_query=query)
+        return (store.get(tiddler) for tiddler in tiddlers)
+    except StoreError, exc:
+        raise FilterIndexRefused('error in the store: %s' % exc)
